-- May your brain not spontaneously explode from the reading of this disorganized mod: Yes, it did.
PhatLoot = LibStub("AceAddon-3.0"):NewAddon("PhatLoot", 'AceEvent-3.0', 'AceHook-3.0')
local L = LibStub("AceLocale-3.0"):GetLocale("PhatLoot", true)

local _G = getfenv(0)

function PhatLoot:OnInitialize()
	self.dbDefaults = {
		profile = {
			scale = 1.0,
			alpha = 1.0,
			cursor = true,
			debug = false,
			smartsnap = true,
			snapoffset = 0,
			altoptions = true,
			collapse = true,
			linkallvis = "always",
			linkallthreshold = 2,
			linkallchannels = "RAID",
			dragborder = true,
			lootexpand = true,
			qualityborder = false,
			qualityframe = false,
			texcolor = true,
			lootqualityborder = true,
			loothighlightframe = true,
			loothighlightthreshold = 1,
			qualitytext = false,
			infotext = true,
			bindtext = true,
			lock = true,
			skipsolobop = true,
			pos = { x = (UIParent:GetWidth()/2), y = (UIParent:GetHeight()/2) },
			bgcolor = { 0, 0, 0, .7 },
			bordercolor = { .7, .7, .7, 1 },
			lootbgcolor = { 0, 0, 0, .9 },
			lootbordercolor = { .5, .5, .5, 1 },
			infocolor = { 1, .8, 0 },
			noscan = false
		}
	}
	self.db = LibStub("AceDB-3.0"):New("PhatLootDB", self.dbDefaults, "global")
	
	--Initial session variables
	self.numButtons = 0 -- Buttons currently created
	self.buttons = {} -- Easy reference array by ID
	self.frames = {}
	self.currentloot = {}
	self.visible = false
	self.open = false
	self.loothasbeenexpanded = false
	self.containershift = false
	self:SetupFrames()
end

--Hook builtin functions
function PhatLoot:OnEnable()
	local db = self.db.profile
	self:Hook("CloseSpecialWindows", true)
	LootFrame:SetScript("OnUpdate", self.LootFrame_Update)
	LootFrame:UnregisterEvent("LOOT_OPENED")
	LootFrame:UnregisterEvent("LOOT_SLOT_CLEARED")
	LootFrame:UnregisterEvent("LOOT_CLOSED")
	self:RegisterEvent("LOOT_OPENED", "OnOpen")
	self:RegisterEvent("LOOT_SLOT_CLEARED", "OnClear")
	self:RegisterEvent("LOOT_CLOSED", "OnClose")
	if self.CreateOptions then
		self:CreateOptions()
	end
end

function PhatLoot:OnDisable()
	self:UnregisterAllEvents()
	LootFrame:RegisterEvent("LOOT_OPENED") 
	LootFrame:RegisterEvent("LOOT_SLOT_CLEARED")
	LootFrame:RegisterEvent("LOOT_CLOSED") 
end

function PhatLoot:Defaults()
	self:Print("Default values restored.")
	for k, v in pairs(self.dbDefaults) do
		self.db.profile[k] = v
	end
end

---- Operational functions/hooks ----
function PhatLoot:CloseSpecialWindows()
	if self.frame:IsShown() then 
		self:AutoClose(true, true)
	end
end

function PhatLoot:OnOpen()
	self:msg("OnOpen()")
	if self:AutoClose() == nil then
		if not self.visible and IsFishingLoot() then
			PlaySound("FISHING REEL IN")
		end
		self:Clear()
		self:Update()
	end
end

function PhatLoot:OnClear()
	self.refreshing = true
	self:Clear()
	self:Update()
end

function PhatLoot:OnClose()
	self:AutoClose()
	StaticPopup_Hide("LOOT_BIND")
	self:Clear()
end

function PhatLoot:OnHide()
	if not self.refreshing then
		self:AutoClose(true)
	else 
		self.refreshing = false
	end
end

function PhatLoot:AutoClose(force) -- Thanks, FruityLoots.
	if (GetNumLootItems() == 0) or force then 
		self:Clear()
		--self:msg("AutoClosing ("..GetNumLootItems() ..")"..(force and " Forced!" or ""))
		--if self.open then
			self:msg("Manually closing the loot frame.")
			--ShowUIPanel(LootFrame)
			HideUIPanel(LootFrame)
			CloseLoot()
			--self.open = false
		--end
		return 1
	end
	self:msg("AutoClose check passed")
	return nil
end

---- Core ----

local function nilTable(tableToNil) -- MentalPower
	if (type(tableToNil) ~= "table") then
		return tableToNil
	end

	for key, value in pairs(tableToNil) do
		if type(value) == "table" then
			nilTable(value)
		end
		tableToNil[key] = nil
	end

	return tableToNil
end

function PhatLoot:Update()
	self:msg("Updating")
	--self.open = true
	local db = self.db.profile
	self.currentloot = nilTable(self.currentloot)
	local numLoot = GetNumLootItems()
	--Build frames if we need more
	if (numLoot > self.numButtons) then
		for i = (self.numButtons + 1), numLoot do
			--self:msg("Adding needed frame["..(i).."], numButtons = "..PhatLoot.numButtons.." & numLoot = "..numLoot)
			self:AddLootFrame(i)
		end
	end
	-- LootLoop
	local slot, curslot, button, frame, texture, item, quantity, quality, color, qualitytext, textobj, infoobj, qualityobj
	local curshift, qualityTower, framewidth  = 0, 0, 0
	for slot = 1, numLoot do
		texture, item, quantity, quality = GetLootSlotInfo(slot)
		if (texture) then
			curshift = curshift +1
			-- If we're shifting loot, use position slots instead of item slots
			if db.collapse then
				button = self.buttons[curshift]
				frame = self.frames[curshift]
				curslot = curshift
			else
				button = self.buttons[slot]
				frame = self.frames[slot]
				curslot = slot
			end
			button:SetID(slot)
			button.slot = slot
			--self:msg("Attaching loot["..slot.."] ["..item.."] to slot ["..curslot.."], bSlot = "..button.slot);
			color = ITEM_QUALITY_COLORS[quality]
			qualityTower = max(qualityTower, quality)
			SetItemButtonTexture(button, texture)
			textobj = _G["PhatLootButton"..curslot.."Text"]
			infoobj = _G["PhatLootButton"..curslot.."Description"]
			qualityobj = _G["PhatLootButton"..curslot.."Quality"]
			infoobj:SetText("")
			infoobj:SetVertexColor(unpack(db.infocolor))
			qualityobj:SetText("")
			if LootSlotIsCoin(slot) then -- Fix and performance fix thanks to Dead_LAN
				item = string.gsub(item, "\n", " ", 1, true);
			end
			
			table.insert(self.currentloot, { texture = texture, item = item, quantity = quantity, quality = quality, link = GetLootSlotLink(slot) })
			
			if db.lootexpand then
				textobj:SetWidth(700)
				infoobj:SetWidth(700)
			else
				textobj:SetWidth(155)
				infoobj:SetWidth(155)
			end
			textobj:SetVertexColor(color.r, color.g, color.b);
			textobj:SetText(item);
			
			if db.qualitytext and not LootSlotIsCoin(slot) then 
				qualityobj:SetText(_G["ITEM_QUALITY"..quality.."_DESC"])
				qualityobj:SetVertexColor(.8, .8, .8, 1);
				textobj:SetPoint("TOPLEFT", button, "TOPLEFT", 42, -12)
				infoobj:SetPoint("TOPLEFT", button, "TOPLEFT", 45, -22)
				textobj:SetHeight(10)
			elseif LootSlotIsCoin(slot) then
				textobj:SetPoint("TOPLEFT", button, "TOPLEFT", 42, 2)
				qualityobj:SetText("")
				button.bind:SetText("")
				textobj:SetHeight(PhatLootButton1:GetHeight()+1)
			else
				qualityobj:SetText("")
				if db.infotext then
					textobj:SetPoint("TOPLEFT", button, "TOPLEFT", 42, -8)
				else
					textobj:SetPoint("TOPLEFT", button, "TOPLEFT", 42, -12)
					infoobj:SetText("")
				end
				infoobj:SetPoint("TOPLEFT", button, "TOPLEFT", 45, -18)
				textobj:SetHeight(10)
			end
			
			if db.lootqualityborder then
				frame:SetBackdropBorderColor(color.r, color.g, color.b, 1)
				button.wrapper:SetBackdropBorderColor(color.r, color.g, color.b, 1)
			else
				frame:SetBackdropBorderColor(unpack(db.lootbordercolor))
				button.wrapper:SetBackdropBorderColor(unpack(db.lootbordercolor))
			end
			
			if LootSlotIsItem(slot) and quality >= db.loothighlightthreshold then
				local r, g, b, hex = GetItemQualityColor(quality)
				if db.texcolor then
					button.border:SetVertexColor(r, g, b)
					button.border:Show()
				else button.border:Hide() end
				if db.loothighlightframe then
					frame.border:SetVertexColor(r, g, b)
					frame.border:Show()
				else frame.border:Hide() end
			else
				button.border:Hide()
				frame.border:Hide()
			end
			
			if LootSlotIsItem(slot) and db.infotext then
				self:SetSlotInfo(slot, button)
			end
			
			if db.lootexpand then 
				framewidth = max(framewidth, textobj:GetStringWidth(), infoobj:GetStringWidth())
			end
			
			SetItemButtonCount(button, quantity)
			button.quality = quality
			button:Show()
			frame:Show()
			
		elseif not db.collapse then
			curshift = curshift + 1
			self.buttons[slot]:Hide()
			--self:msg("Hiding slot "..slot..", curshift: "..curshift)
		end
	end
	
	--if slot == curshift then --Collapse lower buttons
	--	curshift = curshift -1
	--	--self:msg("Collapsing end slot "..slot..", curshift now "..curshift)
	--end
	
	PhatLootFrame:SetScale(db.scale)
	local color = ITEM_QUALITY_COLORS[qualityTower]
	if db.qualityborder and not self.visible then 
		--self:msg("Quality tower: "..qualityTower)
		self.frame:SetBackdropBorderColor(color.r, color.g, color.b, 1)
	else
		 self.frame:SetBackdropBorderColor(unpack(db.bordercolor))
	end
	if db.qualityframe and not self.visible then
		self.frame:SetBackdropColor(color.r, color.g, color.b, db.bgcolor[4])
	else
		self.frame:SetBackdropColor(unpack(db.bgcolor))
	end
		
	PhatLootFrame:SetHeight(20 + (curshift*(PhatLootButtonFrame1:GetHeight()+2)))
	
	if db.lootexpand then
		self.loothasbeenexpanded = true
		local fwidth, bwidth = (self.buttons[1]:GetWidth() + framewidth + 21), -(framewidth + 16)
		self:UpdateWidths(curshift, fwidth, bwidth, fwidth+24)
	else --if self.loothasbeenexpanded then
		self.loothasbeenexpanded = false
		self:UpdateWidths(table.getn(self.frames), 200, -163, 222)
	end
	
	
	if (db.collapse and db.cursor) or (not self.visible and db.cursor) then -- FruityLoot
		self:PositionAtCursor()
	end
	
	self.frame:Show()
	if db.linkallvis == "always" or (db.linkallvis == "raid" and GetNumRaidMembers() > 0) or (db.linkallvis == "party" and GetNumPartyMembers() > 0) then
		self.linkbutton:Show()
	else
		self.linkbutton:Hide()
	end
	--self:msg("Displaying at position: "..PhatLootFrame:GetLeft().." "..PhatLootFrame:GetTop());
	self.visible = true
	
	--Hopefully avoid non-looting/empty bar
	if self:AutoClose() then
		--self:msg("Possible hanger frame. Closing.. "..numLoot..", "..curshift)
	end
end

 function PhatLoot:UpdateWidths(framenum, fwidth, bwidth, ofwidth)
 	local expand = self.db.profile.lootexpand
	for i = 1, framenum do
		self.frames[i]:SetWidth(fwidth)
		self.buttons[i]:SetHitRectInsets(0, bwidth, 0, -1)
		self:QualityBorderResize(self.frames[i])
	end
	self.frame:SetWidth(ofwidth)
end 

function PhatLoot:SetSlotInfo(slot, button) -- Yay wowwiki demo
	local link =  GetLootSlotLink(slot)
	if not link then return nil end -- Avoid errors for now.
	local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc = GetItemInfo(link)
	local oldLoc = itemEquipLoc
	self:SetBindText(self:GetBindOn(itemLink), button.bind)
	if itemType == "Weapon" then
		itemEquipLoc = "Weapon"
	else
		itemEquipLoc = _G[itemEquipLoc]
	end
	if itemSubType == "Junk" then 
		itemSubType = (itemRarity > 0) and L["Miscellaneous"] or itemSubType
	end
--	if type(itemEquipLoc) == "table" then
--		itemEquipLoc = ("BUG itemType, oldLoc = %s, %s"):format(itemType, oldLoc)
--	end
	button.desc:SetText(((type(itemEquipLoc) == "string" and itemEquipLoc ~= "") and itemEquipLoc..", " or "") .. ((itemSubType == itemSubType) and itemSubType or itemSubType.." "..itemType))
end

function PhatLoot:PositionAtCursor() --Fruityloots mixup, only called if cursor snapping is enabled
	x, y = GetCursorPosition()
	local s = PhatLootFrame:GetEffectiveScale()
	x = (x / s) - 30
	y = (y / s) + 30
	local screenWidth = GetScreenWidth()
	if (UIParent:GetWidth() > screenWidth) then screenWidth = UIParent:GetWidth() end
	local screenHeight = GetScreenHeight()
	local windowWidth = PhatLootFrame:GetWidth()
	local windowHeight = PhatLootFrame:GetHeight()
	if (x + windowWidth) > screenWidth then x = screenWidth - windowWidth end
	if y > screenHeight then y = screenHeight end
	if x < 0 then x = 0 end
	if (y - windowHeight) < 0 then y = windowHeight end
	LootFrame:ClearAllPoints()
	if (self.db.profile.smartsnap and self.visible) then
		x = PhatLootFrame:GetLeft()
	else 
		x = x + self.db.profile.snapoffset
	end
	PhatLootFrame:ClearAllPoints()
	PhatLootFrame:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", x, y)
end

-- Add a single lootframe
function PhatLoot:AddLootFrame(id)
	local frame = CreateFrame("Frame", "PhatLootButtonFrame"..id, self.frame)
	local bname = "PhatLootButton"..id
	local button = CreateFrame(LootButton1:GetObjectType(), bname, frame, "LootButtonTemplate")
	-- Equivalent of PhatLootButtonTemplate
	local text = _G[bname.."Text"]
	local desc = button:CreateFontString(bname.."Description")
	local quality = button:CreateFontString(bname.."Quality")
	local bind = button:CreateFontString(bname.."Bind")
	
	local font = { STANDARD_TEXT_FONT, 10, "" }
	
	text:SetDrawLayer("OVERLAY")
	desc:SetDrawLayer("OVERLAY")
	quality:SetDrawLayer("OVERLAY")
	bind:SetDrawLayer("OVERLAY")
	
	desc:SetFont(unpack(font))
	quality:SetFont(unpack(font))
	font[2] = 9
	font[3] = "OUTLINE"
	bind:SetFont(unpack(font))
	
	desc:SetJustifyH("LEFT")
	quality:SetJustifyH("LEFT")
	bind:SetJustifyH("LEFT")
	
	desc:SetHeight(10)	
	quality:SetHeight(10)
	text:SetHeight(10)
	bind:SetHeight(10)
	
	quality:SetWidth(155)
	
	quality:SetPoint("TOPLEFT", button, "TOPLEFT", 45, -3)
	bind:SetPoint("BOTTOMLEFT",  button, "BOTTOMLEFT", 3, 3)

	button:SetHitRectInsets(0, -165, 0, -1)
	-- End template
	local border = self:QualityBorder(button)
	local fborder = self:QualityBorder(frame)
	button.wrapper = self:ItemButtonWrapper(button, 6, 6)
	fborder:SetHeight(fborder:GetHeight() -3)
	fborder:SetPoint("CENTER", frame, "CENTER", 4, .5)
	fborder:SetAlpha(0.3)
	frame:SetWidth(200)
	frame:SetHeight(button:GetHeight()+1)
	button:ClearAllPoints()
	frame:ClearAllPoints()
	if (id == 1) then 
		frame:SetPoint("TOPLEFT", self.frame, "TOPLEFT", 10, -10)
	else
		frame:SetPoint("TOPLEFT", self.frames[id-1], "BOTTOMLEFT", 0, -2)
	end
	button:SetPoint("LEFT", frame, "LEFT")
	button:RegisterForDrag("LeftButton")
	button:SetScript("OnDragStart", function() self:DragStart() end)
	button:SetScript("OnDragStop", function() self:DragStop() end)
	button:SetScript("OnEnter", 	function() local slot = this:GetID() if LootSlotIsItem(slot) then GameTooltip:SetOwner(this, "ANCHOR_RIGHT") GameTooltip:SetLootItem(slot) if IsShiftKeyDown() then GameTooltip_ShowCompareItem() end CursorUpdate() end end )
	button:SetScript("OnUpdate", function() CursorOnUpdate() end)
	self.buttons[id] = button
	self.buttons[id].border = border
	self.frames[id] = frame
	self.frames[id].border = fborder
	--self:msg("Creation: self.buttons["..id.."] = ".. button:GetName())
	self.frame:SetHeight(self.frame:GetHeight() + frame:GetHeight())

	--Skin
	self:Skin(frame)
	
	button.text = text
	button.desc = desc
	button.bind = bind
	button.quality = quality
	button:DisableDrawLayer("ARTWORK")
	button:Hide()
	frame:Hide()
	self.numButtons = self.numButtons +1
end

function PhatLoot:DragStart() 
	if self.db.profile.lock then 
		PhatLootFrame:StartMoving() 
	end
end

function PhatLoot:DragStop()
	if self.db.profile.lock then 
		PhatLootFrame:StopMovingOrSizing()
		self.db.profile.pos.x = PhatLootFrame:GetLeft()
		self.db.profile.pos.y = PhatLootFrame:GetTop()
		--PhatLoot:msg("Setting position: "..self.db.profile.pos.x.." "..self.db.profile.pos.y) 
	end
end 

-- Setup lootframes & close button
function PhatLoot:SetupFrames()
	-- Alright you XML nazis. Main frame
	self.frame = CreateFrame("Frame", "PhatLootFrame", UIParent)
	self.frame:SetFrameStrata("DIALOG")
	self.frame:SetFrameLevel(5)
	self.frame:SetWidth(222)
	self.frame:SetHeight(20)
	self.frame:SetMovable(1)
	if self.db.profile.dragborder then
		self.frame:EnableMouse(1)
	end
	self.frame:RegisterForDrag("LeftButton")
	self.frame:SetScript("OnDragStart", function() PhatLoot:DragStart() end)
	self.frame:SetScript("OnDragStop", function() PhatLoot:DragStop() end)
	self.frame:SetScript("OnHide", function() PhatLoot:OnHide() end)
	--self.frame:IsToplevel(1)
	self:BackdropFrame(self.frame, self.db.profile.bgcolor, self.db.profile.bordercolor)
   	self.frame:ClearAllPoints()
   	if not self.db.profile.cursor then
		self.frame:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", self.db.profile.pos.x, self.db.profile.pos.y)
	end

	--Skin
	self:Skin(PhatLootFrame)
   
   self.frame:SetScale(self.db.profile.scale)
   self.frame:SetAlpha(self.db.profile.alpha)
    
   	-- Close button
	self.closebutton = CreateFrame("Button", "PhatLootCloseButton", PhatLootFrame)
	self.closebutton:SetScript("OnClick", function() PhatLoot:AutoClose(true, true); end)
	self.closebutton:SetFrameLevel(8)
	self.closebutton:SetWidth(32)
	self.closebutton:SetHeight(32)
	self.closebutton:SetNormalTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Up")
	self.closebutton:SetPushedTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Down")
	self.closebutton:SetHighlightTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Highlight")
	self.closebutton:ClearAllPoints()
	self.closebutton:SetPoint("TOPRIGHT", PhatLootFrame, "TOPRIGHT", 3, 3)
	self.closebutton:SetHitRectInsets(5, 5, 5, 5)
	self.closebutton:Show()
	
   	-- Link all button
	self.linkbutton = CreateFrame("Button", "PhatLootLinkButton", PhatLootFrame)
	self.linkbutton:SetScript("OnClick", function() self:ClickLinkLoot() end)
	self.linkbutton.text = self.linkbutton:CreateFontString("PhatLootLinkButtonText", "DIALOG", "GameFontNormalSmall")
	self.linkbutton.text:SetText("|c22AAAAAA"..L["Link All Loot"])
	self.linkbutton.text:SetAllPoints(self.linkbutton)
	self.linkbutton:SetFrameLevel(8)
	self.linkbutton:SetWidth(75)
	self.linkbutton:SetHeight(24)
	self.linkbutton:SetHighlightTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Highlight")
	self.linkbutton:ClearAllPoints()
	self.linkbutton:SetPoint("BOTTOMRIGHT", PhatLootFrame, "BOTTOMRIGHT", -4, -3)
	self.linkbutton:SetHitRectInsets(5, 5, 5, 5)
	self.linkbutton.text:Show()
	self.linkbutton:Show()

	self:AddLootFrame(1)
	self.frame:Hide()
end	

function PhatLoot:msg( text )
	if self.db.profile.debug then
		DEFAULT_CHAT_FRAME:AddMessage("|cff7fff7fPhatLoot|r: "..text)
	end
end

function PhatLoot:Clear()
	for slot, button in pairs(self.buttons) do
		SetItemButtonCount(button, 0)
		button:Hide()
		self.frames[slot]:Hide()
	end
	if GetNumLootItems() < 1 then
		self.visible = false
	end
	PhatLootFrame:Hide()
end

function PhatLoot:LinkToName(link)
	if not link then return nil end
	return string.gsub(link,"^.-%[(.*)%].*", "%1")
end

function PhatLoot:ClickLinkLoot()
	local channel = self.db.profile.linkallchannels
	local linked
	if channel then
		linked = self:LinkLoot(channel)
	end
--[[
	local linked
	if channels then
		for k, v in pairs(channels) do 
			if v then
				linked = self:LinkLoot(k, v.extchannel)
			end
		end
	end
	]]
end

function PhatLoot:LinkLoot(channel, isExtraChannel)
	local output, key, buffer = { }, 1
	
	if UnitExists("target") then
		output[1] = UnitName("target")..":"
	end
	
	local linkthreshold, thresholdreached = self.db.profile.linkallthreshold, false
	for k, v in pairs(self.currentloot) do
		if v.quality >= linkthreshold then
			thresholdreached = true
			buffer = (output[key] and output[key].." " or "")..(v.quantity > 1 and v.quantity.."x" or "")..(v.quantity == 0 and v.item or v.link)
			if strlen(buffer) > 255 then 
				key = key + 1
				output[key] = (v.quantity > 1 and v.quantity.."x" or "")..v.link
			else
				output[key] = buffer
			end
		end
	end
	
	if not thresholdreached then
		return false
	end
	
	local chattype, channelout
	if isExtraChannel then 
		chattype = "CHANNEL"
		channelout = GetChannelName(channel)
	else
		chattype = channel
		channelout = nil
	end
	
	for k, v in pairs(output) do
		v  = string.gsub(v, "\n", " ", 1, true) -- DIE NEWLINES, DIE A HORRIBLE DEATH 
		SendChatMessage(v, chattype, nil, channelout)
	end
	
	return true
end

local bop, boe, bou
function PhatLoot:SetBindText(bind, text)
	if not bop then
		bop, boe, bou = L["BoP"].." ", L["BoE"].." ", L["BoU"].." "
	end
	if bind == "pickup" then
		text:SetText(bop)
		text:SetVertexColor(1, .3, .1)
	elseif bind == "equip" then
		text:SetText(boe)
		text:SetVertexColor(.3, 1, .3)
	elseif bind == "BOU" then
		text:SetText(bou)
		text:SetVertexColor(.3, .5, 1)
	else text:SetText("")	end
end
