local L = LibStub("AceLocale-3.0"):GetLocale("PhatLoot", true)
local PhatLoot = LibStub("AceAddon-3.0"):GetAddon("PhatLoot")

local options = {
	name = "PhatLoot",
	type = "group",
	childGroups = "tab",
	get = function(info)
		return PhatLoot.db.profile[info[#info]]
	end,
	set = function(info, value)
		PhatLoot.db.profile[info[#info]] = value
	end,
	args = {
		frame = {
			name = L["Frame"],
			desc = L["Frame settings."],
			type = "group",
			args = {
				group1 = {
					name = L["Frame Behaviour"],
					type = "group",
					inline = true,
					order = 1,
					args = {
						cursor = {
							name = L["Snap to Cursor"],
							desc = L["Positions the loot frame at the cursor for easy looting."],
							type = "toggle",
							order = 1,
						},
						smartsnap = {
							name = L["Snap Vertically Only"],
							desc = L["Loot frame is only snapped vertically  to the cursor when looting, so it doesn't jump around."],
							type = "toggle",
							order = 2,
						},
						lock = {
							name = L["Draggable"],
							desc = L["Enables dragging of the loot frame."],
							type = "toggle",
							order = 3,
						},
					}
				},
				group2 = {
					name = L["Frame Appearance"],
					type = "group",
					inline = true,
					order = 2,
					args = {
						scale = {
							name = L["Scale"],
							desc = L["Sets the scale of the loot frame."],
							type = "range",
							min = 0.2,
							max = 2,
							step = 0.1,
							order = 4,
						},
						qualityborder = {
							name = L["Color Border By Quality"],
							desc = L["Colors the loot frame border in the color of the highest quality item."],
							type = "toggle",
							order = 5,
						},
						qualityframe = {
							name = L["Color Frame By Quality"],
							desc = L["Colors the loot frame in the color of the highest quality item."],
							type = "toggle",
							order = 6,
						},
					}
				}
			}
		},
		loot = {
			name = L["Loot"],
			desc = L["Loot settings."],
			type = "group",
			args = {
				group1 = {
					name = L["Loot Behaviour"],
					type = "group",
					inline = true,
					order = 1,
					args = {			
						collapse = {
							name = L["Hide Empty Loot"],
							desc = L["Hide looted (empty) buttons and place cursor over the next item if 'Snap to Cursor' is enabled."],
							type = "toggle",
							order = 1,
						},
						lootexpand = {
							name = L["Resize to Fit Loot Width"],
							desc = L["Resizes the loot frame to fit the lootable items."],
							type = "toggle",
							order = 2,
						},
						linkallchannels = {
							name = L["Link to Channel"],
							desc = L["Sets the channel the 'Link All Loot' button links the current loot to."],
							type = "select",
							order = 3,
							values = {RAID=L["Raid"], PARTY=L["Party"], GUILD=L["Guild"],OFFICER=L["Officer"],SAY=L["Say"],WHISPER=L["Whisper"]}
						},
					}
				},
				group2 = {
					name = L["Loot Appearance"],
					type = "group",
					inline = true,
					order = 2,
					args = {
						texcolor = {
							name = L["Color Loot Icon By Quality"],
							desc = L["Colors the loot frame border in the color of the highest quality item."],
							type = "toggle",
							order = 4,
							width = "double",
						},
						lootqualityborder = {
							name = L["Color Loot Border By Quality"],
							desc = L["Colors the loot frame in the color of the highest quality item."],
							type = "toggle",
							order = 5,
							width = "double",
						},	
					}
				}
			}
		}
	}
}

function PhatLoot:CreateOptions()
	LibStub("AceConfig-3.0"):RegisterOptionsTable("PhatLoot", options)
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("PhatLoot", "PhatLoot")
end